* computeMoments.do
* Computes all macro moments for paper and organizes data for VAR analysis of
*  monetary policy shocks

#delimit ;

clear all;
set maxvar 10000;

local dir "C:\...\Empirical Macro";

matrix macromoments = J(7,1,0);
matrix rownames macromoments = "sigma(dlog c)" "sigma(dlog x)" "4r" "4(re-r)" "sigma(dlog y)" "sigma(dlog l)" "sigma(d/p)";

* Compute first and second moments of returns;
use "`dir'\data\monthly.dta", replace;
gen ann_lev_realyield1mo = 12*lev_realyield1mo;
gen ann_lev_excessequity1mo = 12*lev_excessequity1mo;
gen quarter_dp12_dt = 3*dp12_dt;
tabstat ann_lev_realyield1mo ann_lev_excessequity1mo, stat(mean) save;
matrix macromoments[3,1] = el(r(StatTotal),1,1);
matrix macromoments[4,1] = el(r(StatTotal),1,2);
tabstat quarter_dp12_dt if month == 3 | month == 6 | month == 9 | month == 12, stat(sd) save; 
matrix macromoments[7,1] = el(r(StatTotal),1,1);

* Compute second moments of quantities;
use "`dir'\data\quarterly.dta", replace;
qui gen gdpdef = us_gdp/us_rgdp;
qui gen c_pc = (us_cn + us_cs)/(us_pop*gdpdef);
qui gen dlog_c_pc = 100*(log(c_pc) - log(L.c_pc));
qui gen x_pc = (us_fi + us_cd)/(us_pop*gdpdef);
qui gen dlog_x_pc = 100*(log(x_pc) - log(L.x_pc));
qui gen y_pc = us_rgdp/us_pop;
qui gen dlog_y_pc = 100*(log(y_pc) - log(L.y_pc));
qui gen l_pc = us_h/us_pop;
qui gen dlog_l_pc = 100*(log(l_pc) - log(L.l_pc));
tabstat dlog_c_pc dlog_x_pc dlog_y_pc dlog_l_pc, stat(sd) save; 
matrix macromoments[1,1] = el(r(StatTotal),1,1);
matrix macromoments[2,1] = el(r(StatTotal),1,2);
matrix macromoments[5,1] = el(r(StatTotal),1,3);
matrix macromoments[6,1] = el(r(StatTotal),1,4);

* Save moments;
putexcel set "`dir'\output\macromoments.xlsx", replace;
putexcel A1 = matrix(macromoments), rownames;

* Organize and export data for VAR analysis;
use "`dir'\data\monthly.dta", replace;
export delimited year month realyield1mo excessequity1mo relbill 
 dyield3mo termspread dp_dt gs1 logcpi logip ebp mortg_spread_m cp3m_spread_m cm5yr cm10yr
 mp1_tc	ff4_tc cumrelfundreturn_dt cumrelmret_dt using
 "`dir'\data\var.csv", replace;
 
export delimited year month mp1_tc ff4 using "`dir'\data\iv.csv" if ym >= ym(1991,1), replace; 
