function [irf,sp,sq] = genIRFIV(perIRF,varSigma,varResid,ivData,ivNames,mapIVToVar,lags,A,otherVars,iv)
% Generates impulse response functions using external instrument,
%  following Gertler-Karadi (2015)
% Must be the case that first column of varResid is the monetary indicator
%  instrumented with iv!

numOtherVars = size(otherVars,2);
numVars = numOtherVars+1;

ivIndep = [ivData(:,find(strcmp(iv,ivNames),1)),ones(size(ivData,1),1)];
ivBeta = ivIndep\varResid(mapIVToVar-lags,1);
varResidMPHat = ivIndep*ivBeta;

scaledCoeffs = nan(length(otherVars),1);
for i=1:length(otherVars)
    thisBeta = [varResidMPHat,ones(size(varResidMPHat,1),1)]\varResid(mapIVToVar-lags,i+1);
    scaledCoeffs(i) = thisBeta(1);
end

varSigma11 = varSigma(1,1);
varSigma21 = varSigma(2:(length(otherVars)+1),1);
varSigma22 = varSigma(2:(length(otherVars)+1),2:(length(otherVars)+1));

Q = scaledCoeffs*varSigma11*(scaledCoeffs') ...
    - (varSigma21*scaledCoeffs' + scaledCoeffs*varSigma21') ...
    + varSigma22;
sp = sqrt(varSigma11 - (varSigma21 - scaledCoeffs*varSigma11)'*(Q^(-1))*...
    (varSigma21 - scaledCoeffs*varSigma11));
% assume positive sign convention
sq = scaledCoeffs*sp;

T = perIRF;
irf = nan(T-lags+1,numVars);
endo = zeros(T-lags+1,numVars*lags);
irf(1,1) = sp;
irf(1,2:(length(otherVars)+1)) = sq';
endo(1,1) = sp;
endo(1,2:(length(otherVars)+1)) = sq';
for t=2:T
    endo(t,:) = (A*endo(t-1,:)')';
    irf(t,:) = endo(t,1:numVars);
end