function [irf,uExcessDecompMP] = runRobustness(numVars,numOtherVars,lags,...
        varData,ivData,varNames,ivNames,mpInd,otherVars,yName,rName,iv,...
        T,mapIVToVar,rho)
% Runs VAR, estimates effects of monetary policy shock using SVAR-IV,
%  and computes Campbell-Shiller decomposition used in robustness

%% Run VAR

varInds = nan(1,numVars);
varInds(1) = find(strcmp(mpInd,varNames),1);
for j=1:numOtherVars
    varInds(1+j) = find(strcmp(otherVars{j},varNames),1);
end

varDep = varData(lags+1:size(varData,1),varInds);
varIndep = lagmatrix(varData(:,varInds),1:lags);
varIndep = varIndep(lags+1:size(varIndep,1),:);

sy = zeros(1,numVars*lags);
sy(find(strcmp(yName,otherVars),1)+1) = 1;
sr = zeros(1,numVars*lags);
sr(find(strcmp(rName,otherVars),1)+1) = 1;
sd = zeros(1,numVars*lags);

% Run VAR
varBeta = [varIndep,ones(size(varIndep,1),1)]\varDep;
varResid = varDep - [varIndep,ones(size(varIndep,1),1)]*varBeta;
varSigma = (1/(size(ivData,1)-(lags*numVars+1)))*varResid(mapIVToVar-lags,:)'*varResid(mapIVToVar-lags,:);

% Stack VAR into a first-order system
% Let A be the matrix of coefficients on non-constants, in order of thisDep 
%  then thisIndep
A = nan(lags*numVars,lags*numVars);
A(1:numVars,:) = varBeta(1:(size(varBeta,1)-1),:)';
A(numVars+1:lags*numVars,:) = [eye((lags-1)*numVars),zeros((lags-1)*numVars,numVars)];

%% Generate impulse responses using external IV and decompose contemporaneous effects on excess returns

[irf,sp,sq] = genIRFIV(T,varSigma,varResid,ivData,ivNames,mapIVToVar,lags,A,otherVars,iv);

% Decompose contemporaneous effects of MP *loosening* on excess returns
uExcessDecompMP = nan(1,4);
 % [unexpected S&P 500 return, -revised expectation of future excess
 % returns, -revised expectation of future real rates, revised
 % expectation of dividends]
uExcessDecompMP(1) = -irf(1,sy==1)-irf(1,sr==1);
uExcessDecompMP(2) = sy*rho*A*((eye(numVars*lags)-rho*A)^(-1))*[sp;sq;zeros((lags-1)*numVars,1)];
uExcessDecompMP(3) = sr*rho*A*((eye(numVars*lags)-rho*A)^(-1))*[sp;sq;zeros((lags-1)*numVars,1)];
uExcessDecompMP(4) = uExcessDecompMP(:,1) - uExcessDecompMP(:,2) - uExcessDecompMP(:,3);