* computeMoments0709.do
* Summarizes distribution of household portfolios and labor/wealth ratios
*  in 2007 SCF, and then change in labor income through 2009

#delimit;

clear all;
set maxvar 10000;

timer clear 1;
timer on 1;

local dir "C:\...\Empirical Micro";

local al_exvh_pct = 60; 
 * defines percentile of wealth to labor income distribution to cut at;
local rsh_exvh_pct = 90; 
 * defines percentile of portfolio share distribution to cut at, among high wealth to labor income group;
scalar indHet = 1;
 * if = 1, allow for idiosyncratic heterogeneity in leverage (default: 1);
local numDraws = 20;
 * number of idiosyncratic draws for each household;
 * (only relevant if indHet = 1); 
local targetVolAgg = 1.81/5.8;
 * target ratio of standard deviation gross leverage to aggregate firm leverage (matching estimates of Bach-Calvet-Sodini (2018));
 * (only relevant if indHet = 1);
local gammaShape = 0.86018227;
 * initial value of shape parameter for Gamma distribution;

local rsh_exvh_lbound = -10000;
 * ignore observations with rsh_exvh < lbound;
local rsh_exvh_ubound = 10000;
 * ignore observations with rsh_exvh > ubound;

local firmnetlev07 = 1.4;
local firmnetlev09 = 1.3;
* source: Q2 2019 release of FA;
local actbusnetlev07 = 1.1;
local actbusnetlev09 = 1.2;
* source: Q2 2019 release of FA for 2016;

use "`dir'\data\scf0709.dta", replace;
gen db09 = db07; 
 * set estimated DB wealth equal to 2007 wealth, in the absence of revised estimate;

* Draw individual leverage to target targetVolAgg (if indlev = 1);
gen indfirmnetlev07 = `firmnetlev07';
gen indactbusnetlev07 = `actbusnetlev07';

replace WGT09 = WGT09/`numDraws';
expand `numDraws', gen(newFlag);

set seed 1000;
gen udraw = runiform();
local volAgg07 = `targetVolAgg';

foreach year in 07 09 {;
	scalar continueDecomp = 1;
		
	while continueDecomp {;
		if `year' == 07 & indHet {;
			local gammaShape = `gammaShape' - 0.5*(1-`volAgg07'/`targetVolAgg');
			local firmScale07 = `firmnetlev07'/`gammaShape';
			local actbusScale07 = `actbusnetlev07'/`gammaShape';
			qui replace indfirmnetlev07 = `firmScale07'*invgammap(`gammaShape',udraw);
			qui replace indactbusnetlev07 = `actbusScale07'*invgammap(`gammaShape',udraw);
		};
		if `year' == 09 & indHet {;
			local firmScale09 = `firmnetlev09'/`gammaShape';
			local actbusScale09 = `actbusnetlev09'/`gammaShape';		
			gen indfirmnetlev09 = `firmScale09'*invgammap(`gammaShape',udraw);
			gen indactbusnetlev09 = `actbusScale09'*invgammap(`gammaShape',udraw);
		};
		if `year' == 09 & ~indHet {;
			gen indfirmnetlev09 = `firmnetlev09';
			gen indactbusnetlev09 = `actbusnetlev09';
		};

		* Compute positions in capital and bonds for each household;
		gen EQUITY`year'_K = EQUITY`year'*indfirmnetlev`year';
		gen EQUITY`year'_B = EQUITY`year' - EQUITY`year'_K;
		
		gen ACTBUS`year'_K = ACTBUS`year'*indactbusnetlev`year';
		gen ACTBUS`year'_B = ACTBUS`year' - ACTBUS`year'_K;
		
		gen NONACTBUS`year'_K = NONACTBUS`year'*indactbusnetlev`year';
		gen NONACTBUS`year'_B = NONACTBUS`year' - NONACTBUS`year'_K;	
		
		gen VEHIC`year'_B = 0;
		gen VEHIC`year'_K = VEHIC`year';

		gen HOUSES`year'_B = 0;
		gen HOUSES`year'_K = HOUSES`year';

		gen ORESRE`year'_B = 0;
		gen ORESRE`year'_K = ORESRE`year';

		gen NNRESRE`year'_B = 0;
		gen NNRESRE`year'_K = NNRESRE`year';

		gen OTHNFIN`year'_B = 0;
		gen OTHNFIN`year'_K = OTHNFIN`year';
		
		gen db`year'_B = db`year';
		gen db`year'_K = 0;
		
		* Compute total wealth, capital, and bonds for each household;
		gen K`year' = EQUITY`year'_K + ACTBUS`year'_K + NONACTBUS`year'_K + 
		 VEHIC`year'_K + HOUSES`year'_K + ORESRE`year'_K + NNRESRE`year'_K + OTHNFIN`year'_K;
		gen A`year' = NETWORTH`year' + db`year';
		gen B`year' = A`year' - K`year';		
		
		* Compute total *gross* wealth for each household;
		gen A`year'_gross = A`year' + MRTHEL`year' + RESDBT`year' + OTHLOC`year' + CCBAL`year' + INSTALL`year' + ODEBT`year';
		
		* Compute standard deviation of leverage, relative to leverage of aggregate business sector;
		qui gen rsh`year'_gross = K`year'/A`year'_gross;
		qui tabstat rsh`year'_gross [aw=WGT09], statistics(sd) save;
		local volAgg`year' = el(r(StatTotal),1,1)/`firmnetlev`year'';
		di `gammaShape';
		di `volAgg`year'';

		if `year' == 07 {;
			if abs(1-`volAgg07'/`targetVolAgg') > 0.0001 & indHet {;
				drop *_B *_K A`year'* B`year'* K`year'* *_gross;
			};
			else {;
				scalar continueDecomp = 0;
				gen K`year'_exvh = EQUITY`year'_K + ACTBUS`year'_K + NONACTBUS`year'_K + 
					ORESRE`year'_K + NNRESRE`year'_K + OTHNFIN`year'_K;
				gen A`year'_exvh = (NETWORTH`year' - (VEHIC`year' + HOUSES`year') + MRTHEL`year' + VEHINST`year') + db`year';
				gen B`year'_exvh = A`year'_exvh - K`year'_exvh;					
			};
		};
		else {;
			scalar continueDecomp = 0;
			gen K`year'_exvh = EQUITY`year'_K + ACTBUS`year'_K + NONACTBUS`year'_K + 
				ORESRE`year'_K + NNRESRE`year'_K + OTHNFIN`year'_K;
			gen A`year'_exvh = (NETWORTH`year' - (VEHIC`year' + HOUSES`year') + MRTHEL`year' + VEHINST`year') + db`year';
			gen B`year'_exvh = A`year'_exvh - K`year'_exvh;				
		};		
	};
};

* Generate portfolio shares and wealth/labor measures;
foreach theend in "" "_exvh" {;
	foreach theyear in 07 09 {;
		gen rsh`theyear'`theend' = K`theyear'`theend' / A`theyear'`theend';
		gen al`theyear'`theend' = A`theyear'`theend' / WAGEINC`theyear';
		* replace with large number if wageinc = 0;
		replace al`theyear'`theend' = 1000000000 if WAGEINC`theyear' == 0;
		gen la`theyear'`theend' = WAGEINC`theyear' / A`theyear'`theend';
	};
};

* Define first group in 07;
_pctile al07_exvh [aw=WGT09], percentiles(`al_exvh_pct');
scalar al_exvh_bound = r(r1);
gen al_exvh_group = 0 if al07_exvh < al_exvh_bound;
replace al_exvh_group = 1 if al07_exvh >= al_exvh_bound;
 
* Within high group, define groups of portfolio share distribution;
_pctile rsh07_exvh if al_exvh_group == 1 [aw=WGT09], percentiles(`rsh_exvh_pct');
scalar rsh_exvh_bound = r(r1);
gen rsh_exvh_group = 0 if rsh07_exvh < rsh_exvh_bound & rsh07_exvh >= `rsh_exvh_lbound';
replace rsh_exvh_group = 1 if rsh07_exvh >= rsh_exvh_bound & rsh07_exvh <= `rsh_exvh_ubound';

* Define composite groups (0 = low group1, 1 = high wealth to labor income, low portfolio share, 2 = high wealth to labor income, high portfolio share);
gen group = 0 if al_exvh_group == 0;
replace group = 1 + rsh_exvh_group if al_exvh_group == 1 & rsh_exvh_group < .;

* Define change in key variables between 2007 and 2009;
gen dWAGEINC = WAGEINC09 - WAGEINC07;
gen dA = A09 - A07;
gen dK = K09 - K07;

* Summarize characteristics of groups;
matrix summStats = (0,0,0\0,0,0\0,0,0\0,0,0\0,0,0\0,0,0);
tabstat WGT09, by(group) statistics(sum) format(%9.3f) save;
matrix summStats[1,1] = el(r(Stat1),1,1)/el(r(StatTotal),1,1);
matrix summStats[1,2] = el(r(Stat2),1,1)/el(r(StatTotal),1,1);
matrix summStats[1,3] = el(r(Stat3),1,1)/el(r(StatTotal),1,1);

tabstat WAGEINC07 [aw=WGT09], by(group) statistics(sum) format(%9.3f) save;
matrix summStats[2,1] = el(r(Stat1),1,1)/el(r(StatTotal),1,1);
matrix summStats[2,2] = el(r(Stat2),1,1)/el(r(StatTotal),1,1);
matrix summStats[2,3] = el(r(Stat3),1,1)/el(r(StatTotal),1,1);
	
tabstat A07 [aw=WGT09], by(group) statistics(sum) format(%9.3f) save;
matrix summStats[3,1] = el(r(Stat1),1,1)/el(r(StatTotal),1,1);
scalar total1 = el(r(Stat1),1,1);
matrix summStats[3,2] = el(r(Stat2),1,1)/el(r(StatTotal),1,1);
scalar total2 = el(r(Stat2),1,1);
matrix summStats[3,3] = el(r(Stat3),1,1)/el(r(StatTotal),1,1);	
scalar total3 = el(r(Stat3),1,1);

tabstat K07 [aw=WGT09], by(group) statistics(sum) format(%9.3f) save;
matrix summStats[4,1] = el(r(Stat1),1,1)/total1;
matrix summStats[4,2] = el(r(Stat2),1,1)/total2;
matrix summStats[4,3] = el(r(Stat3),1,1)/total3;

tabstat A09 [aw=WGT09], by(group) statistics(sum) format(%9.3f) save;
matrix summStats[5,1] = el(r(Stat1),1,1)/el(r(StatTotal),1,1) - summStats[3,1];
scalar total1 = el(r(Stat1),1,1);
matrix summStats[5,2] = el(r(Stat2),1,1)/el(r(StatTotal),1,1) - summStats[3,2];
scalar total2 = el(r(Stat2),1,1);
matrix summStats[5,3] = el(r(Stat3),1,1)/el(r(StatTotal),1,1) - summStats[3,3];	
scalar total3 = el(r(Stat3),1,1);

tabstat K09 [aw=WGT09], by(group) statistics(sum) format(%9.3f) save;
matrix summStats[6,1] = el(r(Stat1),1,1)/total1;
matrix summStats[6,2] = el(r(Stat2),1,1)/total2;
matrix summStats[6,3] = el(r(Stat3),1,1)/total3;	

matrix rownames summStats = "Frac households" "07 Frac wageinc" 
	"07 Frac A" "07 K/A" "09 - 07 Frac A" "09 K/A"; 		
matrix colnames summStats = "c" "b" "a";

putexcel set "`dir'\output\heterogeneity0709.xlsx", replace;
putexcel A1 = matrix(summStats), names;

* Project rsh09 on rsh07;
regress rsh09 rsh07 if abs(rsh07) <= 10 [aw=WGT09], robust;

timer off 1;
timer list 1;
