%% plotSensitivity2016.m
% Summarizes wealth shares and portfolio shares as risky share cutoff varies

sensitivityFile = importdata('output/sensitivity2016.csv');
sensitivityData = sensitivityFile.data;
sensitivityNames = sensitivityFile.colheaders; 

Interpreter = 'latex';
fontsize = 14;
% Define colors
lightblue   = [107,174,214]./255;
darkblue    = [8,69,148]./255;

groupNames = {'Group $c$','Group $b$','Group $a$'};
varNames = {'$\sum_{i \in c} W \ell^i / \sum_i W \ell^i$','$\sum_{i \in c} A^i / \sum_i A^i$','$\sum_{i \in c} Q k^i / \sum_{i \in c} A^i$';...
	'$\sum_{i \in b} W \ell^i / \sum_i W \ell^i$','$\sum_{i \in b} A^i / \sum_i A^i$','$\sum_{i \in b} Q k^i / \sum_{i \in b} A^i$';...
	'$\sum_{i \in a} W \ell^i / \sum_i W \ell^i$','$\sum_{i \in a} A^i / \sum_i A^i$','$\sum_{i \in a} Q k^i / \sum_{i \in a} A^i$'};

fig = figure;

color_list = {darkblue, lightblue};
color_list_bw = {rgb2gray(darkblue), rgb2gray(lightblue)};
for bw=0:1
    if bw
        this_color_list = color_list_bw;
    else
        this_color_list = color_list;
    end
        
    subplot(1,3,1);
    Wl_group1_3 = plot(sensitivityData(:,find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1)),sensitivityData(:,find(strcmp(sensitivityNames,strcat('fracWl_',num2str(3))),1)),'LineWidth',2,'Color',this_color_list{1},'Linestyle','-');
    hold on;
    Wl_group1_2 = plot(sensitivityData(:,find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1)),sensitivityData(:,find(strcmp(sensitivityNames,strcat('fracWl_',num2str(2))),1)),'LineWidth',2,'Color',this_color_list{2},'Linestyle','-');
    hold off;
    xlim([sensitivityData(1,find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1)),sensitivityData(size(sensitivityData,1),find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1))]);
    ylim([0,1]);
    title('$\sum_{i \in g} W \ell^i / \sum_i W \ell^i$','Fontsize',fontsize,'Interpreter',Interpreter);
    ax2 = gca;
    ax2.XTick = [50 75 99];
    ax2.XTickLabel = {'$p50$','$p75$','$p99$'};
    ax2.FontSize = fontsize;
    ax2.TickLabelInterpreter = Interpreter;  
    xlabel('$Qk^i/A^i$','Fontsize',fontsize,'Interpreter',Interpreter);
    box on;    

    subplot(1,3,2);
    A_group1_3 = plot(sensitivityData(:,find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1)),sensitivityData(:,find(strcmp(sensitivityNames,strcat('fracA_',num2str(3))),1)),'LineWidth',2,'Color',this_color_list{1},'Linestyle','-');
    hold on;
    A_group1_2 = plot(sensitivityData(:,find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1)),sensitivityData(:,find(strcmp(sensitivityNames,strcat('fracA_',num2str(2))),1)),'LineWidth',2,'Color',this_color_list{2},'Linestyle','-');
    hold off;
    xlim([sensitivityData(1,find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1)),sensitivityData(size(sensitivityData,1),find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1))]);
    ylim([0,1]);    
    title('$\sum_{i \in g} A^i / \sum_i A^i$','Fontsize',fontsize,'Interpreter',Interpreter);
    ax2 = gca;
    ax2.XTick = [50 75 99];
    ax2.XTickLabel = {'$p50$','$p75$','$p99$'};    
    ax2.FontSize = fontsize;
    ax2.TickLabelInterpreter = Interpreter;
    xlabel('$Qk^i/A^i$','Fontsize',fontsize,'Interpreter',Interpreter);
    box on;    

    subplot(1,3,3);
    KA_group1_3 = plot(sensitivityData(:,find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1)),sensitivityData(:,find(strcmp(sensitivityNames,strcat('fracKA_',num2str(3))),1)),'LineWidth',2,'Color',this_color_list{1},'Linestyle','-');
    hold on;
    KA_group1_2 = plot(sensitivityData(:,find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1)),sensitivityData(:,find(strcmp(sensitivityNames,strcat('fracKA_',num2str(2))),1)),'LineWidth',2,'Color',this_color_list{2},'Linestyle','-');
    hold off;
    xlim([sensitivityData(1,find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1)),sensitivityData(size(sensitivityData,1),find(strcmp(sensitivityNames,'rsh_exvh_pct_vec1'),1))]);
    ylim([0,5]);
    title('$\sum_{i \in g} Q k^i / \sum_{i \in g} A^i$','Fontsize',fontsize,'Interpreter',Interpreter);    
    legend([KA_group1_3,KA_group1_2],{'$g = a$','$g = b$'},'Fontsize',fontsize-2,'Interpreter',Interpreter,'Location','northwest');
    ax2 = gca;
    ax2.XTick = [50 75 99];
    ax2.XTickLabel = {'$p50$','$p75$','$p99$'};    
    ax2.FontSize = fontsize;
    ax2.TickLabelInterpreter = Interpreter;
    xlabel('$Qk^i/A^i$','Fontsize',fontsize,'Interpreter',Interpreter);
    box on;

    set(gcf,'PaperType','usletter','PaperOrientation','portrait','PaperPosition',[0 0 8.5 2.75]);
    if bw
        print(gcf, 'output/sensitivity2016_bw', '-depsc', '-r600','-loose');  
    else
        print(gcf, 'output/sensitivity2016', '-depsc', '-r600','-loose');  
    end
    close all;
end